@@ -8,6 +8,7 @@
 
 #include "config.h"
 #include "buildrun.h"
+#include "session.h"
 
 #include <fstream>
 #include <sstream>
@@ -16,6 +17,11 @@ extern "C" {
 #include "signal.h"
 #include <sys/wait.h>
 #include <pwd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <string.h>
+#include <errno.h>
 }
 
 
@@ -81,9 +87,20 @@ compile_pass (systemtap_session& s)
 
   o.close ();
 
-  // Run make
+  // Generate module directory pathname and make sure it exists.
   string module_dir = string("/lib/modules/")
     + s.kernel_release + "/build";
+  struct stat st;
+  rc = stat(module_dir.c_str(), &st);
+  if (rc != 0)
+    {
+	clog << "Module directory " << module_dir << " check failed: "
+	     << strerror(errno) << endl
+	     << "Make sure kernel devel is installed." << endl;
+	return rc;
+    }  
+
+  // Run make
   string make_cmd = string("make")
     + string (" -C \"") + module_dir + string("\"");
   make_cmd += string(" M=\"") + s.tmpdir + string("\" modules");
@@ -108,33 +125,31 @@ run_pass (systemtap_session& s)
   struct passwd *pw = getpwuid(getuid());
   string username = string(pw->pw_name);
 
-  // for now, just spawn stpd
-  string stpd_cmd = string("sudo ") 
-    + string(PKGLIBDIR) + "/stpd "
-    + (s.bulk_mode ? "" : "-r ")
+  // for now, just spawn staprun
+  string staprun_cmd = string("sudo ") 
+    + string(BINDIR) + "/staprun "
     + (s.verbose>1 ? "" : "-q ")
-    + (s.merge ? "" : "-m ")
     + "-u " + username + " "
     + (s.output_file.empty() ? "" : "-o " + s.output_file + " ");
   
-  stpd_cmd += "-d " + stringify(getpid()) + " ";
+  staprun_cmd += "-d " + stringify(getpid()) + " ";
   
   if (s.cmd != "")
-    stpd_cmd += "-c \"" + s.cmd + "\" ";
+    staprun_cmd += "-c \"" + s.cmd + "\" ";
   
   if (s.target_pid)
-    stpd_cmd += "-t " + stringify(s.target_pid) + " ";
+    staprun_cmd += "-t " + stringify(s.target_pid) + " ";
   
   if (s.buffer_size)
-    stpd_cmd += "-b " + stringify(s.buffer_size) + " ";
+    staprun_cmd += "-b " + stringify(s.buffer_size) + " ";
   
-  stpd_cmd += s.tmpdir + "/" + s.module_name + ".ko";
+  staprun_cmd += s.tmpdir + "/" + s.module_name + ".ko";
   
-  if (s.verbose>1) clog << "Running " << stpd_cmd << endl;
+  if (s.verbose>1) clog << "Running " << staprun_cmd << endl;
   
   signal (SIGHUP, SIG_IGN);
   signal (SIGINT, SIG_IGN);
-  rc = system (stpd_cmd.c_str ());
+  rc = system (staprun_cmd.c_str ());
 
   return rc;
 }
