@@ -14,6 +14,7 @@
 #include "elaborate.h"
 #include "translate.h"
 #include "buildrun.h"
+#include "session.h"
 
 #include <iostream>
 #include <fstream>
@@ -138,6 +139,7 @@ main (int argc, char * const argv [])
   s.cmd = "";
   s.target_pid = 0;
   s.merge=true;
+  s.perfmon=0;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
   if (s_p != NULL)  
@@ -300,23 +302,24 @@ main (int argc, char * const argv [])
   int rc = 0;
 
   // override PATH and LC_ALL
-  char* path = "PATH=/bin:/sbin:/usr/bin:/usr/sbin";
-  char* lc_all = "LC_ALL=C";
-  rc = putenv (path) || putenv (lc_all);
+  const char *path = "/bin:/sbin:/usr/bin:/usr/sbin";
+  rc = setenv("PATH", path, 1) || setenv("LC_ALL", "C", 1);
   if (rc)
     {
       const char* e = strerror (errno);
-      cerr << "setenv (\"" << path << "\" + \"" << lc_all << "\"): "
+      cerr << "setenv (\"PATH=" << path << "\" + \"LC_ALL=C\"): "
            << e << endl;
     }
 
+  s.kernel_base_release.assign(s.kernel_release, 0, s.kernel_release.find('-'));
+
 
   // arguments parsed; get down to business
 
   // Create a temporary directory to build within.
   // Be careful with this, as "s.tmpdir" is "rm -rf"'d at the end.
   {
-    char* tmpdir_env = getenv("TMPDIR");
+    const char* tmpdir_env = getenv("TMPDIR");
     if (! tmpdir_env)
       tmpdir_env = "/tmp";
     
@@ -365,9 +368,8 @@ main (int argc, char * const argv [])
   version_suffixes.push_back ("/" + kvr + "/" + arch);
   version_suffixes.push_back ("/" + kvr);
   // add kernel version (2.6.NN) + arch
-  string::size_type dash_index = kvr.find ('-');
-  if (dash_index > 0 && dash_index != string::npos) {
-    kvr.erase(dash_index);
+  if (kvr != s.kernel_base_release) {
+    kvr = s.kernel_base_release;
     version_suffixes.push_back ("/" + kvr + "/" + arch);
     version_suffixes.push_back ("/" + kvr);
   }
