@@ -27,10 +27,10 @@ class Bench
     if @@printed_header == 0
       print_header
     end
-    if @@stpd.nil?
-      for path in ['/usr/libexec/systemtap/stpd', '/usr/local/libexec/systemtap/stpd'] 
+    if @@staprun.nil?
+      for path in ['/usr/bin/staprun', '/usr/local/bin/staprun'] 
 	if File.exist?(path)
-	  @@stpd = path
+	  @@staprun = path
 	  break
 	end
       end
@@ -67,8 +67,8 @@ class Bench
       sum=0
       threads.times {|cpu| fork {exec "./itest #{threads} > #{@dir}/bench#{cpu}"}}
       threads.times {Process.waitpid(-1)}	# wait for itest(s) to exit
-      `sudo killall -HUP stpd`
-      Process.wait	# wait for stpd to exit
+      `sudo killall -HUP staprun`
+      Process.wait	# wait for staprun to exit
       threads.times {|x| sum = sum + `cat #{@dir}/bench#{x}`.split[0].to_i - @@ftime}
       @results[threads] = sum / (threads * threads)
       File.open("#{@dir}/xxx.out") do |file|
@@ -115,12 +115,14 @@ class Bench
 
   @@ftime = 0
   @@printed_header = 0
-  @@stpd = nil
+  @@staprun = nil
   @@runtime = nil
   @@num_threads = []
   @@minfreq = 0
 
   def cleanup
+    system('sudo killall -HUP staprun &> /dev/null')
+    system('sudo /sbin/rmmod bench &> /dev/null')
     `/bin/rm -f stap.out` if File.exists?("stap.out")
     `/bin/rm -f bench.stp` if File.exists?("bench.stp")
     `/bin/rm -rf #{@dir}` unless @dir.nil?
@@ -128,9 +130,9 @@ class Bench
   end
 
   def load
-    args = "-rmq -b 8"
-    if @trans == RELAYFS then args = "-mq" end
-    fork do exec "sudo #{@@stpd} #{args} #{@dir}/bench.ko > #{@dir}/xxx 2> #{@dir}/xxx.out" end
+    args = "-q -b 8"
+    if @trans == RELAYFS then args = "-q" end
+    fork do exec "sudo #{@@staprun} #{args} #{@dir}/bench.ko > #{@dir}/xxx 2> #{@dir}/xxx.out" end
     sleep 5
   end
 
@@ -169,6 +171,7 @@ int probe_start(void)\n{\n  return _stp_register_kprobes (kp, NUM_KPROBES);\n}\n
 void probe_exit (void)\n{\n  _stp_unregister_kprobes (kp, NUM_KPROBES); \n}\n"
       rescue
 	puts "Error writing source file"
+	exit
       ensure
 	f.close unless f.nil?
       end
@@ -250,7 +253,7 @@ class Stapbench < Bench
   def load
     # we do this in several steps because the compilation phase can take a long time
     args = "-kvvp4"
-    if @trans == RELAYFS then args = "-bkvvp4" end
+    if @trans == RELAYFS then args = "-bMkvvp4" end
     res = `stap #{args} -m bench bench.stp &> stap.out`
     if $? != 0
       puts "ERROR running stap\n#{res}"
@@ -265,9 +268,9 @@ class Stapbench < Bench
       cleanup
       exit
     end
-    args = "-rmq -b 8"
-    if @trans == RELAYFS then args = "-mq" end
-    fork do exec "sudo #{@@stpd} #{args} #{@dir}/bench.ko > #{@dir}/xxx 2> #{@dir}/xxx.out" end
+    args = "-q -b 8"
+    if @trans == RELAYFS then args = "-q" end
+    fork do exec "sudo #{@@staprun} #{args} #{@dir}/bench.ko > #{@dir}/xxx 2> #{@dir}/xxx.out" end
     sleep 5
   end
   
