@@ -1,5 +1,5 @@
 /*
- * libstp - stpd 'library'
+ * libstp - staprun 'library'
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,8 +38,18 @@
 #include <sys/socket.h>
 #include <linux/types.h>
 #include <linux/limits.h>
+#include <sys/wait.h>
+#include <sys/statfs.h>
+#include <stdint.h>
 #include "librelay.h"
 
+/* stp_check script */
+#ifdef PKGLIBDIR
+char *stp_check=PKGLIBDIR "/stp_check";
+#else
+char *stp_check="stp_check";
+#endif
+
 /* maximum number of CPUs we can handle - change if more */
 #define NR_CPUS 256
 
@@ -48,6 +58,7 @@ static struct params
 {
 	unsigned subbuf_size;
 	unsigned n_subbufs;
+	int merge;
 	char relay_filebase[256];
 } params;
 
@@ -68,8 +79,6 @@ static int transport_mode;
 static int ncpus;
 static int print_totals;
 static int exiting;
-static int processing;
-static pthread_mutex_t processing_mutex;
 
 /* per-cpu data */
 static int relay_file[NR_CPUS];
@@ -81,7 +90,7 @@ static pthread_t reader[NR_CPUS];
 static int control_channel;
 
 /* flags */
-extern int print_only, quiet, merge, verbose;
+extern int print_only, quiet, verbose;
 extern unsigned int buffer_size;
 extern char *modname;
 extern char *modpath;
@@ -167,6 +176,7 @@ static void close_relayfs_files(int cpu)
 	
 	munmap(relay_buffer[cpu], total_bufsize);
 	close(relay_file[cpu]);
+	relay_file[cpu] = 0;
 	fclose(percpu_tmpfile[cpu]);
 }
 
@@ -265,12 +275,22 @@ static int kill_percpu_threads(int n)
 		if (pthread_cancel(reader[i]) == 0)
 			killed++;
 	}
-	if (killed != n)
-		fprintf(stderr, "WARNING: couldn't kill all per-cpu threads:  %d killed, %d total\n", killed, n);
 
 	return killed;
 }
 
+/**
+ *     wait_for_percpu_threads - wait for all threads to exit
+ *     @n: number of threads to wait for
+ */
+static void wait_for_percpu_threads(int n)
+{
+	int i;
+
+	for (i = 0; i < n; i++)
+		pthread_join(reader[i], NULL);
+}
+
 /**
  *	process_subbufs - write ready subbufs to disk
  */
@@ -333,10 +353,6 @@ static void *reader_thread(void *data)
 
 		rc = read(proc_file[cpu], &status[cpu].info,
 			  sizeof(struct buf_info));
-		pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
-		pthread_mutex_lock(&processing_mutex);
-		processing++;
-		pthread_mutex_unlock(&processing_mutex);
 		subbufs_consumed = process_subbufs(&status[cpu].info);
 		if (subbufs_consumed) {
 			if (subbufs_consumed > status[cpu].max_backlog)
@@ -347,34 +363,13 @@ static void *reader_thread(void *data)
 			if (write (proc_file[cpu], &consumed_info, sizeof(struct consumed_info)) < 0)
 				fprintf(stderr,"WARNING: writing consumed info failed.\n");
 		}
-		pthread_mutex_lock(&processing_mutex);
-		processing--;
-		pthread_mutex_unlock(&processing_mutex);
-		pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+		if (status[cpu].info.flushing)
+			pthread_exit(NULL);
 	} while (1);
 }
 
-static void read_last_buffers(void)
-{
-	int cpu, rc;
-	struct consumed_info consumed_info;
-	unsigned subbufs_consumed;
-
-	for (cpu = 0; cpu < ncpus; cpu++) {
-		rc = read (proc_file[cpu], &status[cpu].info, sizeof(struct buf_info));
-		subbufs_consumed = process_subbufs(&status[cpu].info);
-		if (subbufs_consumed) {
-			if (subbufs_consumed > status[cpu].max_backlog)
-				status[cpu].max_backlog = subbufs_consumed;
-			status[cpu].info.consumed += subbufs_consumed;
-			consumed_info.cpu = cpu;
-			consumed_info.consumed = subbufs_consumed;
-			if (write (proc_file[cpu], &consumed_info, sizeof(struct consumed_info)) < 0)
-				fprintf(stderr,"WARNING: writing consumed info failed.\n");
-		}
-	}
-}
-
+#define RELAYFS_MAGIC			0xF0B4A981
+#define DEBUGFS_MAGIC			0x64626720
 /**
  *	init_relayfs - create files and threads for relayfs processing
  *
@@ -382,9 +377,37 @@ static void read_last_buffers(void)
  */
 int init_relayfs(void)
 {
-	int i, j;
+	int i, j, wstat;
+	pid_t pid;
+	struct statfs st;
+
 	dbug("initializing relayfs\n");
 
+	/* first run the _stp_check script */
+	if ((pid = fork()) < 0) {
+		perror ("fork of stp_check failed.");
+		exit(-1);
+	} else if (pid == 0) {
+		if (execlp(stp_check, stp_check, NULL) < 0)
+			_exit (-1);
+	}
+	if (waitpid(pid, &wstat, 0) < 0) {
+		perror("waitpid");
+		exit(-1);
+	}
+	if (WIFEXITED(wstat) && WEXITSTATUS(wstat)) {
+		perror (stp_check);
+		fprintf(stderr, "Could not execute %s\n", stp_check);
+		return -1;
+	}
+
+	if (statfs("/mnt/relay", &st) == 0 && (int) st.f_type == (int) RELAYFS_MAGIC)
+ 		sprintf(params.relay_filebase, "/mnt/relay/systemtap/%d/cpu", getpid());
+ 	else if (statfs("/sys/kernel/debug", &st) == 0 && (int) st.f_type == (int) DEBUGFS_MAGIC)
+ 		sprintf(params.relay_filebase, "/sys/kernel/debug/systemtap/%d/cpu", getpid());
+ 	else
+ 		sprintf(params.relay_filebase, "/debug/systemtap/%d/cpu", getpid());
+	
 	for (i = 0; i < ncpus; i++) {
 		if (open_relayfs_files(i, params.relay_filebase) < 0) {
 			fprintf(stderr, "ERROR: couldn't open relayfs files, cpu = %d\n", i);
@@ -468,12 +491,11 @@ static void cleanup_and_exit (int);
 
 /**
  *	init_stp - initialize the app
- *	@relay_filebase: full path of base name of the per-cpu relayfs files
  *	@print_summary: boolean, print summary or not at end of run
  *
  *	Returns 0 on success, negative otherwise.
  */
-int init_stp(const char *relay_filebase, int print_summary)
+int init_stp(int print_summary)
 {
 	char buf[1024];
 	struct transport_info ti;
@@ -505,9 +527,6 @@ int init_stp(const char *relay_filebase, int print_summary)
 		fprintf(stderr, "ERROR, couldn't insmod probe module %s\n", modpath);
 		return -1;
 	}
-
-	if (relay_filebase)
-		strcpy(params.relay_filebase, relay_filebase);
 	
 	sprintf (proc_filebase, "/proc/systemtap/%s", modname);
 	char *ptr = index(proc_filebase,'.');
@@ -519,7 +538,7 @@ int init_stp(const char *relay_filebase, int print_summary)
 	control_channel = open(buf, O_RDWR);
 	if (control_channel < 0) {
 		fprintf(stderr, "ERROR: couldn't open control channel %s: errcode = %s\n", buf, strerror(errno));
-		return -1;
+		goto do_rmmod;
 	}
 
 	/* start target_cmd if necessary */
@@ -532,31 +551,37 @@ int init_stp(const char *relay_filebase, int print_summary)
 	ti.n_subbufs = 0;
 	ti.target = target_pid;
 	if (send_request(STP_TRANSPORT_INFO, &ti, sizeof(ti)) < 0) {
-		fprintf(stderr, "stpd failed because TRANSPORT_INFO returned an error.\n");
+		fprintf(stderr, "staprun failed because TRANSPORT_INFO returned an error.\n");
 		if (target_cmd)
 			kill (target_pid, SIGKILL);
 		close(control_channel);
-		return -1;
+		goto do_rmmod;
 	}
 	return 0;
+
+do_rmmod:
+	snprintf(buf, sizeof(buf), "/sbin/rmmod -w %s", modname);
+	if (system(buf))
+		fprintf(stderr, "ERROR: couldn't rmmod probe module %s.\n", modname);
+	return -1;
 }
 
 
 /* length of timestamp in output field 0 */
-#define TIMESTAMP_SIZE (sizeof(int))
+#define TIMESTAMP_SIZE (sizeof(uint32_t))
 
 /**
  *	merge_output - merge per-cpu output
  *
  */
-#define MERGE_BUF_SIZE 32768
-
+#define MERGE_BUF_SIZE 16*1024
 static int merge_output(void)
 {
-	int c, i, j, dropped=0;
-	long count=0, min, num[ncpus];
-	char buf[32], tmp[PATH_MAX];
+	char *buf[ncpus], tmp[PATH_MAX];
+	int i, j, dropped=0;
+	uint32_t count=0, min, num[ncpus];
 	FILE *ofp, *fp[ncpus];
+	uint32_t length[ncpus];
 
 	for (i = 0; i < ncpus; i++) {
 		sprintf (tmp, "%s%d", percpu_tmpfilebase, i);
@@ -565,10 +590,18 @@ static int merge_output(void)
 			fprintf (stderr, "error opening file %s.\n", tmp);
 			return -1;
 		}
-		if (fread (buf, TIMESTAMP_SIZE, 1, fp[i]))
-			num[i] = *((int *)buf);
-		else
-			num[i] = 0;
+		num[i] = 0;
+		buf[i] = malloc(MERGE_BUF_SIZE);
+		if (!buf[i]) {
+			fprintf(stderr,"Out of memory in merge_output(). Aborting merge.\n");
+			printf("Out of memory in merge_output(). Aborting merge.\n");
+			return -1;
+		}
+
+		if (fread_unlocked (&length[i], sizeof(uint32_t), 1, fp[i])) {
+			if (fread_unlocked (buf[i], length[i]+TIMESTAMP_SIZE, 1, fp[i]))
+				num[i] = *((uint32_t *)buf[i]);
+		}
 	}
 
 	if (!outfile_name)
@@ -591,34 +624,33 @@ static int merge_output(void)
 			}
 		}
 
-		while (1) {
-			c = fgetc_unlocked (fp[j]);
-			if (c == 0 || c == EOF)
-				break;
-			if (!quiet)
-				fputc_unlocked (c, stdout);
-			if (!print_only)
-				fputc_unlocked (c, ofp);
-		}
+		if (min && !quiet)
+			fwrite_unlocked (buf[j]+TIMESTAMP_SIZE, length[j], 1, stdout);
+		if (min && !print_only)
+			fwrite_unlocked (buf[j]+TIMESTAMP_SIZE, length[j], 1, ofp);
+		
 		if (min && ++count != min) {
 			count = min;
 			dropped++ ;
 		}
 
-		if (fread (buf, TIMESTAMP_SIZE, 1, fp[j]))
-			num[j] = *((int *)buf);
-		else
-			num[j] = 0;
+		num[j] = 0;
+		if (fread_unlocked (&length[j], sizeof(uint32_t), 1, fp[j])) {
+			if (fread_unlocked (buf[j], length[j]+TIMESTAMP_SIZE, 1, fp[j]))
+				num[j] = *((uint32_t *)buf[j]);
+		}
 	} while (min);
 
 	if (!print_only)
-		fputs ("\n", ofp);
+		fwrite_unlocked ("\n", 1, 1, ofp);
 
 	for (i = 0; i < ncpus; i++)
 		fclose (fp[i]);
 	fclose (ofp);
+
 	if (dropped)
 		fprintf (stderr, "Sequence had %d drops.\n", dropped);
+
 	return 0;
 }
 
@@ -639,27 +671,17 @@ static void cleanup_and_exit (int closed)
 		fprintf(stderr,"\nWaititing for processes to exit\n");
 	while(wait(NULL) > 0);
 
-	if (transport_mode == STP_TRANSPORT_RELAYFS) {
-		kill_percpu_threads(ncpus);
-		while(1) {
-			pthread_mutex_lock(&processing_mutex);
-			if (!processing) {
-				pthread_mutex_unlock(&processing_mutex);
-				break;
-			}
-			pthread_mutex_unlock(&processing_mutex);
-		}
-		read_last_buffers();
-	}
+	if (transport_mode == STP_TRANSPORT_RELAYFS && relay_file[0] > 0)
+		wait_for_percpu_threads(ncpus);
 
 	close_proc_files();
 
 	if (print_totals && verbose)
 		summarize();
 
-	if (transport_mode == STP_TRANSPORT_RELAYFS) {
+	if (transport_mode == STP_TRANSPORT_RELAYFS && relay_file[0] > 0) {
 		close_all_relayfs_files();
-		if (merge) {
+		if (params.merge) {
 			merge_output();
 			delete_percpu_files();
 		}
@@ -717,7 +739,6 @@ int stp_main_loop(void)
 	FILE *ofp = stdout;
 
 	setvbuf(ofp, (char *)NULL, _IOLBF, 0);
-	pthread_mutex_init(&processing_mutex, NULL);
 
 	signal(SIGINT, sigproc);
 	signal(SIGTERM, sigproc);
@@ -752,22 +773,23 @@ int stp_main_loop(void)
 			transport_mode = info->transport_mode;
 			params.subbuf_size = info->subbuf_size;
 			params.n_subbufs = info->n_subbufs;
+			params.merge = info->merge;
 #ifdef DEBUG
-			if (transport_mode == STP_TRANSPORT_RELAYFS)
-				fprintf (stderr, "TRANSPORT_INFO recvd: RELAYFS %d bufs of %d bytes.\n", 
-					 params.n_subbufs, 
-					 params.subbuf_size);
-			else
-				fprintf (stderr, "TRANSPORT_INFO recvd: PROC with %d Mbyte buffers.\n", 
+			if (transport_mode == STP_TRANSPORT_RELAYFS) {
+				fprintf(stderr,"TRANSPORT_INFO recvd: RELAYFS %d bufs of %d bytes.\n", 
+					params.n_subbufs, 
+					params.subbuf_size);
+				if (params.merge)
+					fprintf(stderr,"Merge output\n");
+			} else
+				fprintf(stderr,"TRANSPORT_INFO recvd: PROC with %d Mbyte buffers.\n", 
 					 info->buf_size); 
 #endif
 			if (!streaming()) {
 				rc = init_relayfs();
 				if (rc < 0) {
-					close(control_channel);
 					fprintf(stderr, "ERROR: couldn't init relayfs, exiting\n");
-					/* FIXME. Need to cleanup properly */
-					exit(1);
+					cleanup_and_exit(0);
 				}
 			} else if (outfile_name) {
 				ofp = fopen (outfile_name, "w");
@@ -799,9 +821,11 @@ int stp_main_loop(void)
 		{
 			struct transport_start *t = (struct transport_start *)data;
 			dbug("probe_start() returned %d\n", t->pid);
-			if (t->pid < 0)
+			if (t->pid < 0) {
+				if (target_cmd)
+					kill (target_pid, SIGKILL);
 				cleanup_and_exit(0);
-			else if (target_cmd)
+			} else if (target_cmd)
 				kill (target_pid, SIGUSR1);
 			break;
 		}
