@@ -9,19 +9,24 @@
 #ifndef SESSION_H
 #define SESSION_H
 
+#include "elaborate.h"
 #include <string>
 #include <vector>
 #include <iostream>
 #include <sstream>
 #include <map>
 
+extern "C" {
+#include <elfutils/libdw.h>
+}
+
 
 // forward decls for all referenced systemtap types
 struct match_node;
 struct stapfile;
 struct vardecl;
 struct functiondecl;
-struct derived_probe;
+struct derived_probe_group_container;
 struct embeddedcode;
 struct translator_output;
 struct unparser;
@@ -62,6 +67,7 @@ struct systemtap_session
   std::vector<std::string> macros;
   std::vector<std::string> args;
   std::string kernel_release;
+  std::string kernel_base_release;
   std::string architecture;
   std::string runtime_path;
   std::string module_name;
@@ -75,7 +81,9 @@ struct systemtap_session
   bool guru_mode;
   bool bulk_mode;
   bool unoptimized;
+  bool merge;
   int buffer_size;
+  unsigned perfmon;
 
   // temporary directory for module builds etc.
   // hazardous - it is "rm -rf"'d at exit
@@ -93,7 +101,7 @@ struct systemtap_session
   std::vector<stapfile*> files;
   std::vector<vardecl*> globals;
   std::vector<functiondecl*> functions;
-  std::vector<derived_probe*> probes;
+  derived_probe_group_container probes;
   std::vector<embeddedcode*> embeds;
   std::map<std::string, statistic_decl> stat_decls;
   // XXX: vector<*> instead please?
@@ -105,6 +113,11 @@ struct systemtap_session
   translator_output* op;
   unparser* up;
 
+  // kprobes_text data
+  bool kprobes_text_initialized;
+  Dwarf_Addr kprobes_text_start;
+  Dwarf_Addr kprobes_text_end;
+
   unsigned num_errors;
   // void print_error (const parse_error& e);
   void print_error (const semantic_error& e);
