@@ -38,6 +38,17 @@ lex_cast_qstring(IN const & in)
 }
 
 
+template <typename OUT, typename IN> inline OUT
+lex_cast(IN const & in)
+{
+  stringstream ss;
+  OUT out;
+  if (!(ss << in && ss >> out))
+    throw runtime_error("bad lexical cast");
+  return out;
+}
+
+
 expression::expression ():
   type (pe_unknown), tok (0)
 {
@@ -91,19 +102,21 @@ symboldecl::~symboldecl ()
 
 probe_point::probe_point (std::vector<component*> const & comps,
 			  const token * t):
-  components(comps), tok(t) 
+  components(comps), tok(t), optional (false)
 {
 }
 
 probe_point::probe_point ():
-  tok (0)
+  tok (0), optional (false)
 {
 }
 
 
+unsigned probe::last_probeidx = 0;
 probe::probe ():
   body (0), tok (0)
 {
+  this->name = string ("probe_") + lex_cast<string>(last_probeidx ++);
 }
 
 
@@ -853,6 +866,8 @@ void probe_point::print (ostream& o) const
       if (c->arg)
         o << "(" << *c->arg << ")";
     }
+  if (optional)
+    o << "?";
 }
 
 
